@@ -114,7 +114,7 @@ use parent qw/Plack::Component/;
 
 binmode STDERR, ":utf8"; 
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 =pod
 
@@ -245,8 +245,10 @@ processing the request.
 
 sub respond {
     my ($self, $responder, $env) = @_;
+    # TODO: logging
     if ($env->{REQUEST_METHOD} eq 'OPTIONS') {
-        $responder->([200, ['Content-Type' => 'text/plain', 
+        $responder->([200, ['Content-Length' => 0,
+                            'Content-Type' => 'text/plain', 
                             Geo::OGC::Service::Common::CORS($self)]]);
     } else {
         my $service;
@@ -300,10 +302,20 @@ error is reported as an XML message using OGC conventions.
 sub service {
     my ($self, $responder, $env) = @_;
 
-    my $service = { env => $env };
-
     my $request = Plack::Request->new($env);
     my $parameters = $request->parameters;
+
+    my $service = { 
+        env => $env, 
+        request => $request,
+        processor => $self->{processor} 
+        # will get below:
+        # posted
+        # filter
+        # parameters
+        # service
+        # config
+    };
     
     my %names;
     for my $key (sort keys %$parameters) {
@@ -370,9 +382,13 @@ sub service {
         $service_from_script_name->($env) // ''; 
 
     if (exists $self->{services}{$requested_service}) {
+        bless $service, $self->{services}{$requested_service};
         $service->{service} = $requested_service;
-        $service->{config} = get_config($self->{config}, $requested_service);
-        return bless $service, $self->{services}{$requested_service};
+        my $config = $self->{config_maker} ?
+            $self->{config_maker}->config($self->{config}) :
+            $self->{config};
+        $service->{config} = get_config($config, $requested_service);
+        return $service;
     }
 
     error($responder, { exceptionCode => 'InvalidParameterValue',
@@ -417,7 +433,7 @@ sub error {
     my $attributes = { exceptionCode => $msg->{exceptionCode} };
     my $content;
     $content = [ ExceptionText => $msg->{ExceptionText} ] if exists $msg->{ExceptionText};
-    if ($msg->{exceptionCode} eq 'MissingParameterValue') {
+    if (exists $msg->{locator}) {
         $attributes->{locator} = $msg->{locator};
     }
     $writer->element('Exception', $attributes, $content);
@@ -598,13 +614,23 @@ convenience for writing XML to the client.
 
 The element method has the syntax
 
-  $writer->xml_element($tag[, $attributes][, $content])
+  $writer->element($tag[, $attributes][, $content])
+
+or
+
+  $writer->element($element)
+
+where $element is a reference to an array [$tag[, $attributes][,
+$content]].
 
 $attributes is a reference to a hash
 
-$content is a reference to a list of xml elements (tag...)
+$content is nothing, undef, '/>', plain content (string), an element
+(as above), a list of elements, or a reference to a list of
+elements. If there is no $content or $content is undef, a self-closing
+tag is written. If $content is '/>' a closing tag is written.
 
-Setting $tag to 1, allows writing plain content.
+Setting $tag to 0 or 1, allows writing plain content.
 
 If $attribute{$key} is undefined the attribute is not written at all.
 
@@ -613,25 +639,29 @@ If $attribute{$key} is undefined the attribute is not written at all.
 package Geo::OGC::Service::XMLWriter;
 use Modern::Perl;
 use Encode qw(decode encode is_utf8);
+use Carp;
 
 sub element {
     my $self = shift;
-    my $element = shift;
+    my $tag = shift;
+    return unless defined $tag;
+    if (ref($tag) eq 'ARRAY') {
+        for my $element ($tag, @_) {
+            $self->element(@$element);
+        }
+        return;
+    }
     my $attributes;
-    my $content;
-    for my $x (@_) {
-        $attributes = $x, next if ref($x) eq 'HASH';
-        $content = $x;
-    }
-    if (defined $content && $content eq '/>') {
-        $self->write("</$element>");
+    $attributes = shift if @_ and ref($_[0]) eq 'HASH';
+    if (@_ && defined($_[0]) && $_[0] eq '/>') {
+        $self->write("</$tag>");
         return;
     }
-    if ($element =~ /^1/) {
-        $self->write($content);
+    if ($tag =~ /^\d/) {
+        $self->write($_[0]);
         return;
     }
-    $self->write("<$element");
+    $self->write("<$tag");
     if ($attributes) {
         for my $a (keys %$attributes) {
             my $attr = $attributes->{$a};
@@ -641,24 +671,23 @@ sub element {
             }
         }
     }
-    unless (defined $content) {
+    if (@_ == 0 || !defined($_[0])) {
         $self->write(" />");
     } else {
         $self->write(">");
-        if (ref $content) {
-            if (ref $content->[0]) {
-                for my $e (@$content) {
-                    $self->element(@$e);
-                }
+        for my $element (@_) {
+            next unless defined($element);
+            if (ref $element eq 'ARRAY') {
+                $self->element(@$element);
+            } elsif (ref $element) {
+                croak ref($element)." can't be used as an XML element.";
+            } elsif ($element eq '>') {
             } else {
-                $self->element(@$content);
+                $element = decode utf8 => $element unless is_utf8($element);
+                $self->write($element);
             }
-            $self->write("</$element>");
-        } elsif ($content eq '>') {
-        } else {
-            $content = decode utf8 => $content unless is_utf8($content);
-            $self->write("$content</$element>");
         }
+        $self->write("</$tag>");
     }
 }
 
